# -*- coding: utf-8 -*-
##############################################################################
#
#    Original Module for OpenERP, Open Source Management Solution
#    Copyright (C) 20014 TeMPO CONSULTING (<http://tempo-consulting.fr>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
from openerp.osv import osv, fields

from tempfile import TemporaryFile
import base64
import threading
import pooler
import logging
import csv
import time


class product_category(osv.Model):
    _inherit = 'product.category'
    
    _import_product_uom_map = {
        '/PC': 'Unit(s)',  # note: 'PCE' in FR
        'HRE': 'Hour(s)',  # note: 'HRE' in FR
        'KG': 'kg',
        'KM': 'km',
        'M': 'm',
        'M2': 'm2',
        'M3': 'm3',
        'L': 'Liter(s)',
    }
    
    _import_product_uom_default = _import_product_uom_map['/PC']
    
    _import_default_product_state = 'sellable'
    
    _import_message_subject = u"Rapport d'import de catégorie d'article '%s'"
            
    def import_file(self, cr, uid, categ_br, file_obj, context=None):
        if context is None:
            context = {}
        context['last_error'] = False       
        ids_map = {}  # { 'model1': { 'name1': id, ... }, ... }
        
        try:
            reader = csv.DictReader(file_obj, delimiter=';',
                quoting=csv.QUOTE_MINIMAL)
                
            index = 1
            imported = 0
            count = 0
            for row in reader:
                if not self._import_check_header(index, row, categ_br,
                    context=context):
                    break
                if self._import_row(cr, uid, row, index, categ_br,
                    ids_map=ids_map, context=context):
                    imported += 1
                else:
                    break
                index += 1
                count += 1
        finally:
            file_obj.close()
            
        if context['last_error']:
            body = u"Erreur lors de l'import de la catégorie '%s': %s" % (
                categ_br.name or '', context['last_error'], )
        else:
            body = u"%d / %d article(s) de la catégorie '%s' importé(s)" % (
                imported, count, categ_br.name or '', )
        print body
        
        # emission du rapport
        subject = self._import_message_subject % (categ_br.name or '')
        self._send_notification(cr, uid, subject=subject, body=body,
            context=context)
        cr.commit()
  
    def _send_notification(self, cr, uid, subject='', body='', context=None):
        user_br = self.pool.get('res.users').browse(cr, uid, uid,
            context=context)
        if not user_br or not user_br.alias_id or \
            not user_br.alias_id.alias_name or not user_br.partner_id: 
            return
  
        vals = {
            'subject': subject,
            'body': body,
            'email_from': user_br.alias_id.alias_name + '@localhost',
            'partner_ids': [(4, user_br.partner_id.id)],
            'date': time.strftime('%Y-%m-%d %H:%M:%S'),
            'type': 'notification',
        }
        self.pool.get('mail.message').create(cr, uid, vals, context=context)
            
    def _import_check_header(self, index, row, categ_br, context=None):
        fields = [
            'default_code',
            'name',
            'PASS',
            'axima_type',
            'UOM',
            'standard_price',
            'list_price',
        ]
        
        if len(row.keys()) != len(fields):
            self._import_add_err(index, categ_br, 'invalid headers',
                context=context)
            return False  # invalid fields count
            
        # missing field ?
        missing = []
        for f in fields:
            if f not in row:
                missing.append(f)
        if missing:
            msg = 'entête invalide, champ manquant: %s' % (",".join(missing))
            self._import_add_err(index, categ_br, msg, context=context)
            return False
        return True

    def _import_row(self, cr, uid, row, index, categ_br, ids_map=None,
        context=None):
        product_obj = self.pool.get('product.product')
        if context is None:
            context = {}
        context['index'] = index
        
        # checks
        name = row.get('name', False)
        if not name:
            self._import_add_err(index, categ_br.name, 'nom manquant',
                context=context)
            return False
        code = row.get('default_code', False)
        if not code:
            self._import_add_err(index, categ_br.name, 'code manquant',
                context=context)
            return False
    
        # get product category id ('categ_id')
        categ_id = categ_br.id
                
        # product exists already ?
        domain = [('default_code', '=', code)]
        product_id = product_obj.search(cr, uid, domain, context=context)
            
        # set vals
        vals = row.copy()
        # name encoding
        vals['name'] = vals['name'].decode('iso-8859-1').encode('utf8')
        # product flagged as imported...
        # ...for axima_record_rule_product_by_import record rule
        self._import_vals_clean(categ_br, vals)
        vals['state'] = self._import_default_product_state
        vals['import_flag'] = 'imported'
        vals['sap_product_code'] = code  # = default_code
        vals['categ_id'] = categ_id
        self._import_vals_encode_floats(vals, ['list_price', 'standard_price'])
        """
        if not 'standard_price' in vals and 'list_price' in vals:
            # default standard_price (pur price) of list_price (sale price)
            vals['standard_price'] = vals['list_price']
        """
        if not self._import_vals_set_uom(cr, uid, vals, ids_map=ids_map,
            context=context):
            return False
        
        context['import_category_products'] = True
        if product_id:
            product_obj.write(cr, uid, product_id, vals, context=context)
            print 'updated, line ', index
        else:
            product_obj.create(cr, uid, vals, context=context)
            print 'created, line ', index
        del context['import_category_products']
        cr.commit()
        return True
    
    def _import_vals_clean(self, categ_br, vals):
        if '' in vals:
            del vals['']
        if 'PASS' in vals:
            del vals['PASS']
        if categ_br.name and categ_br.name == 'Mercuriale':
            # cleaning names with severals weird " (from original 'Mercuriale' xls)
            vals['name'] = vals['name'].replace('"', '')
        
    def _import_vals_encode_floats(self, vals, keys):
        for k in keys:
            if k in vals:
                if vals[k]:
                    value = vals[k]
                    if value == 'NC':
                        del vals[k]
                    else:
                        trim_strs = [ ' ', '\xc2', '\xa0', ]
                        for ts in trim_strs:
                            value = value.replace(ts, '')
                        if ',' in value:
                            value = float(value.replace(',', '.'))
                        elif not '.' in vals[k]:
                            value = float(value + '.0')
                        vals[k] = value
                else:
                    del vals[k]
                
    def _import_vals_set_uom(self, cr, uid, vals, ids_map=None,
        context=None):
        uom_name = False
        if 'UOM' in vals:
            uom_name = self._import_product_uom_map.get(vals['UOM'].upper(),
                False)
            del vals['UOM']
        if not uom_name:
            # mapping uom invalide (versus une valeur par défaut...)
            #uom_name = self._import_product_uom_default
            self._import_add_err(context['index'], False, 'UOM non valide',
                context=context)
            return False
        uom_id = self._import_get_id_from_name(cr, uid, 'product.uom', uom_name,
            ids_map=ids_map, context=context)
        if uom_id:
            vals['uom_id'] = uom_id
            vals['uom_po_id'] = uom_id
        return True
                   
    def _import_get_category_id(self, cr, uid, name, ids_map=None,
        context=None):
        category_name = name.replace('_', ' ')
        return self._import_get_id_from_name(cr, uid, 'product.category',
            category_name, ids_map=ids_map, context=context)
           
    def _import_get_id_from_name(self, cr, uid, model, name, ids_map=None,
        context=None):
        if not model in ids_map:
            ids_map[model] = {}
        id = ids_map[model].get(name, False)
        if id:
            return id
        domain = [('name', '=', name)]
        id = self.pool.get(model).search(cr, uid, domain, context=context)
        if id:
            id = id[0]
        ids_map[model][name] = id
        return id
        
    def _import_add_err(self, index, categ_br, description, context=None):
        if context is None:
            context = {}
        msg = "! ERREUR / Import article '%s' / ligne %d / %s" % (
            categ_br and categ_br.name or '', index + 1, description, )
        print msg
        context['last_error'] = msg
        logging.getLogger('import product category').info(msg)
                    
            
class product_supplier_category_create_wizard(osv.osv_memory):
    """
    Wizard de création d'une catégorie fournisseur
    """
    _name = "product.supplier.category.create.wizard"
    _description = u"Créer catégorie d'article fournisseur - Assistant"
    
    _columns = {
        'name': fields.char('Nom de la catégorie', size=64, required=True),
    }
    
    def _get_user_office_code(self, cr, uid, context=None):
        # get user office code
        user = self.pool.get('res.users').browse(cr, uid, uid, context=context)
        if not user or not user.company_id or not user.company_id.office_code:
            return False
        return user.company_id.office_code
 
    def do(self, cr, uid, ids, context=None):
        oc = self._get_user_office_code(cr, uid, context=context)
        if not oc or oc != 'GFCN':
            raise osv.except_osv("Avertissement",
                u"Uniquement autorisé pour l'agence Contrats Nationaux")
            
        if isinstance(ids, (int, long)):
            ids = [ids]
        wr = self.browse(cr, uid, ids, context=context)[0]
        if not wr:
            return False
            
        product_categ_obj = self.pool.get('product.category')
        ids = product_categ_obj.search(cr, uid,
            [('name', '=', wr.name)], context=context)
        if ids:
            raise osv.except_osv("Avertissement", 
                u"Cette catégorie existe déjà")
            
        vals = {
            'parent_id': 1,  # Tous les articles
            'type': 'normal',
            'name': wr.name,
        }
        categ_id = self.pool.get('product.category').create(cr, 1, vals,
            context=context)
            
        if categ_id:
            # action tree_view des catégories
            action = self.pool.get('ir.actions.act_window').for_xml_id(cr, uid,
                'product', 'product_category_action_form', context=context)
            if action:
                action['domain'] = [('id', '=', categ_id), ]
                return action
            else:
                return False
        else:
            return False
            
            
class product_main_category_update_wizard(osv.osv_memory):
    """
    Wizard de mise à jour d'une catégorie d'article
    """
    _name = "product.main.category.update.wizard"
    _description = u"Mise à jour catégorie d'article - Assistant"
    
    _note = u"Une fois démarré, l'import à lieu en tâche de fond, vous recevrez un message une fois le traitement terminé. (Messagerie/Pour moi (décocher 'A lire'))\n\nFormat du fichier CSV\nSéparateur ;\nEntête obligatoire: default_code;name;PASS;axima_type;UOM;standard_price;list_price\nCorrespond à: code;nom;nom;type axima;unité de mesure;prix d'achat;prix de vente"
    
    _columns = {
        'categ_id': fields.many2one('product.category',
            string='Catégorie',
            domain=[('parent_id', '=', 1), ],
            required=True
        ),
        'file': fields.binary('Fichier CSV', required=True),
        'note': fields.text('Note', readonly=True),
    }
    
    _defaults = {
        'note': _note,
    }
    
    def _get_user_office_code(self, cr, uid, context=None):
        # get user office code
        user = self.pool.get('res.users').browse(cr, uid, uid, context=context)
        if not user or not user.company_id or not user.company_id.office_code:
            return False
        return user.company_id.office_code
    
    def do(self, cr, uid, ids, context=None):
        oc = self._get_user_office_code(cr, uid, context=context)
        if not oc or oc != 'GFCN':
            raise osv.except_osv("Avertissement",
                u"Uniquement autorisé pour l'agence Contrats Nationaux")
        
        if isinstance(ids, (int, long)):
            ids = [ids]
            
        thread = threading.Thread(target=self._import_csv,
            args=(cr.dbname, uid, ids, context))
        thread.start()
        #self._import_csv(cr.dbname, uid, ids, context=context)  # debug mono thread
 
        # action tree_view des catégories
        return self.pool.get('ir.actions.act_window').for_xml_id(cr, uid,
            'product', 'product_category_action_form', context=context)
    
    def _import_csv(self, dbname, uid, ids, context=None):        
        cr = pooler.get_db(dbname).cursor()
        wz_br = self.browse(cr, uid, ids, context=context)[0]
        if not wz_br:
            return False

        file_obj = TemporaryFile('w+')
        file_obj.write(base64.decodestring(wz_br.file))
        file_obj.seek(0)
        self.pool.get('product.category').import_file(cr, uid, wz_br.categ_id,
            file_obj, context=context)
