# -*- coding: utf-8 -*-
##############################################################################
#
#    Original Module for OpenERP, Open Source Management Solution
#    Copyright (C) 20014 TeMPO CONSULTING (<http://tempo-consulting.fr>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
from openerp.osv import osv, fields
from openerp import tools

import ldap
import logging
from ldap.filter import filter_format

import openerp.exceptions
from openerp import SUPERUSER_ID
from openerp.modules.registry import RegistryManager
_logger = logging.getLogger(__name__)
import re


class res_users(osv.osv):
    _inherit = 'res.users'
    
    _columns = {
        'sub_company_address': fields.text(u"Adresse secondaire"),
    }
        
    def hook_ldap_user_create(self, cr, uid, user_id, login, ldap_entry,
        context=None):
        """
        for ldap_entry content:
        check auth_ldap/users_ldap.py: res_users.authenticate()
        """
        if ldap_entry:
            dn = ldap_entry[0]
            if not dn:
                return
            ou = False
            
            """
            split dn and get first OU (last OU node parent of user)
            (office code always here)
            dn: CN=FIRSTNAME Lastname,OU=Montauban-GF12,OU=Montauban2,
            OU=UTILISATEURS,OU=04 - AXIMA REFRIGERATION,OU=GCR,
            DC=D29,DC=tes,DC=local
            """
            nodes = dn.split(',')
            for n in nodes:
                node_parts = n.split('=')
                if len(node_parts) == 2:
                    if node_parts[0] == 'OU':
                        # first OU in dn (last OU NODE)
                        ou = node_parts[1]
                        break
            if ou:
                office_code = self.ldap_get_office_code_from_ou(ou)
                if office_code:
                    self.register_company(cr, uid, user_id, login, office_code,
                        context=context)
    
    def ldap_get_office_code_from_ou(self, ou):
        """
        get axima office code from OU (organisational unit) sufix
        OU is ended by -GF[NN|NNN] pattern (2 or 3 digits)
        'Any AD group name-GFNN' or 'Any AD group name-GFNNN'
        """
        match_obj = re.search('-GF(\d{2,3})$', ou)
        if match_obj is not None:
            office_number = match_obj.group(1)
            return 'GF' + office_number
            
        # special cases 
        if ou.endswith('-GFCN'):
            # 'Contrats Nationaux' office
            return 'GFCN'
            
        return False
    
    def register_company(self, cr, uid, user_id, login, office_code,
        context=None):
        # get company from code
        domain = [
            ('office_code', '=', office_code),
        ]
        comp_id = self.pool.get('res.company').search(cr, uid, domain,
            context=context)
        if not comp_id:
            msg = u"Impossible de lier l'utilisateur au code agence '%s'"
            raise osv.except_osv("Avertissement", msg % (office_code, ))
        comp_id = comp_id[0]
        company_obj = self.pool.get('res.company').browse(cr, uid, comp_id,
            context=context)
            
        # get sale team
        if company_obj and company_obj.name:
            domain = [
                ('name', '=', company_obj.name),
            ]
            ccs_id = self.pool.get('crm.case.section').search(cr, uid, domain,
                context=context)
        
        vals = {
            'company_id': comp_id,
            # (4, ID) link to existing record with id = ID (adds a relationship)
            'company_ids': [(4, comp_id)],
            'default_section_id': ccs_id and ccs_id[0] or False,
        }
        if login:
            # default mail for internal messages, bc even if alias is set,
            # sending internal message need email field filled...
            vals['email'] = login + '@localhost'
        self.write(cr, uid, user_id, vals, context=context)

res_users()


class CompanyLDAP(osv.osv):
    _inherit = 'res.company.ldap'
    
    _columns = {
        'ldap_base': fields.char('LDAP base', size=128, required=True),  # resize from standard 64
    }
    
    def get_or_create_user(self, cr, uid, conf, login, ldap_entry,
                           context=None):
        """
        Retrieve an active resource of model res_users with the specified
        login. Create the user if it is not initially found.

        :param dict conf: LDAP configuration
        :param login: the user's login
        :param tuple ldap_entry: single LDAP result (dn, attrs)
        :return: res_users id
        :rtype: int
        """
        user_id = False
        login = tools.ustr(login.lower())
        cr.execute("SELECT id, active FROM res_users WHERE lower(login)=%s", (login,))
        res = cr.fetchone()
        if res:
            if res[1]:
                user_id = res[0]
        elif conf['create_user']:
            _logger.debug("Creating new OpenERP user \"%s\" from LDAP" % login)
            user_obj = self.pool.get('res.users')
            values = self.map_ldap_attributes(cr, uid, conf, login, ldap_entry)
            if conf['user']:
                values['active'] = True
                user_id = user_obj.copy(cr, SUPERUSER_ID, conf['user'],
                                        default=values)
            else:
                user_id = user_obj.create(cr, SUPERUSER_ID, values)
            # Added by Tempo
            user_obj.hook_ldap_user_create(cr, uid, user_id, login, ldap_entry,
                context=context)
        return user_id
    
CompanyLDAP()

